@@ -21,7 +21,7 @@ import yaml
 
 from pytest_diffeo import redis_address
 import kvlayer
-from kvlayer import MissingID, BadKey
+from kvlayer import BadKey
 import yakonfig
 
 logger = logging.getLogger(__name__)
@@ -78,11 +78,8 @@ def test_basic_storage(client):
 
     client.delete('t1', (u1, u2))
     assert 0 == len(list(client.scan('t1')))
-    with pytest.raises(MissingID):
-        list(client.scan('t1', ((u1,), (u1,))))
-
-    with pytest.raises(MissingID):
-        list(client.scan('t2', ((u2,), (u3,))))
+    assert 0 == len(list(client.scan('t1', ((u1,), (u1,)))))
+    assert 0 == len(list(client.scan('t2', ((u2,), (u3,)))))
 
 def test_delete(client):
     client.setup_namespace({'table1': 1})
@@ -98,10 +95,7 @@ def test_delete(client):
         for key, value in client.get('table1', key):
             assert kv_dict[key] == value
     for key in delete_keys:
-        generator = client.get('table1', key)
-        with pytest.raises(MissingID):
-            row = generator.next()
-            assert not row
+        assert list(client.get('table1', key)) == [(key, None)]
 
 def test_get(client):
     client.setup_namespace(dict(t1=1, t2=2, t3=3))
@@ -141,9 +135,7 @@ def test_adding_tables(client):
     assert 1 == len(list(client.scan('t1')))
     assert 1 == len(list(client.scan('t1', ((u1,), (u1,)))))
     assert 1 == len(list(client.scan('t3', ((u1,), (u1,)))))
-
-    with pytest.raises(MissingID):
-        list(client.scan('t2', ((u2,), (u3,))))
+    assert 0 == len(list(client.scan('t2', ((u2,), (u3,)))))
 
 @pytest.mark.performance
 def test_large_writes(client):
@@ -179,8 +171,7 @@ def test_setup_namespace_idempotent(client):
     client.delete_namespace()
     client.setup_namespace(dict(t1=2))
     assert 0 == len(list(client.scan('t1')))
-    with pytest.raises(MissingID):
-        list(client.scan('t1', ((u1,), (u1,))))
+    assert 0 == len(list(client.scan('t1', ((u1,), (u1,)))))
 
 
 def test_storage_speed(client):
@@ -298,6 +289,32 @@ def test_scan(client):
                       client.scan('t1', ((),(uuid.UUID(int=75),)))]
     assert actual_results == expected_results
 
+def test_scan_2d(client):
+    '''Set up a table with keys x,y and scan a single known x.'''
+    client.setup_namespace({'t1': 2})
+    def kf(x, y): return (uuid.UUID(int=x), uuid.UUID(int=y))
+    def k1f(x): return (uuid.UUID(int=x),)
+    def vf(x, y): return '{}:{}'.format(x, y)
+    def kvf(x, y): return (kf(x, y), vf(x, y))
+    def scan(fr, to): return list(client.scan('t1', (fr, to)))
+    r = range(1,5) # [1, 2, 3, 4]
+
+    kvps = [kvf(x, y) for x in r for y in r]
+    client.put('t1', *kvps)
+
+    assert list(client.scan('t1')) == kvps
+    assert scan(k1f(2), k1f(2)) == [kvf(2,y) for y in r]
+    assert scan(kf(2,1), kf(2,2)) == [kvf(2,1), kvf(2,2)]
+    assert scan(kf(2,0), kf(2,3)) == [kvf(2,1), kvf(2,2), kvf(2,3)]
+    assert scan(kf(2,2), kf(2,5)) == [kvf(2,2), kvf(2,3), kvf(2,4)]
+    assert scan(kf(2,5), kf(3,0)) == []
+    assert scan(kf(2,4), kf(3,1)) == [kvf(2,4), kvf(3,1)]
+    assert scan(k1f(4), k1f(6)) == [kvf(4,y) for y in r]
+    assert scan(k1f(0), k1f(1)) == [kvf(1,y) for y in r]
+    assert scan(k1f(0), k1f(6)) == kvps
+    assert scan(kf(0,6), kf(6,0)) == kvps
+    assert scan(k1f(0), k1f(0)) == []
+
 def test_no_keys(client):
     """Test that standard kvlayer APIs work correctly when not passed keys"""
     client.setup_namespace({'t1': 1})
